;; Generated package description from site-lisp.el  -*- no-byte-compile: t -*-
(define-package "site-lisp" "0.3.0" "Manage site-lisp directories" '((emacs "25.1")) :commit "87656ba13c805285e50e7d86c29e444272b7d74f" :authors '(("Philip Kaludercic" . "philipk@posteo.net")) :maintainer '("Philip Kaludercic" . "philipk@posteo.net") :keywords '("lisp" "local") :url "https://codeberg.org/pkal/site-lisp.el")
